/*
  The interfrost dtheta_dpressure evaluator is an algebraic evaluator of a given model.
Interfrost water content portion sl.  
  Generated via evaluator_generator.
*/

#include "interfrost_dtheta_dpressure_evaluator.hh"
#include "interfrost_dtheta_dpressure_model.hh"

namespace Amanzi {
namespace Flow {
namespace Relations {

// Constructor from ParameterList
InterfrostDthetaDpressureEvaluator::InterfrostDthetaDpressureEvaluator(Teuchos::ParameterList& plist) :
    SecondaryVariableFieldEvaluator(plist)
{
  Teuchos::ParameterList& sublist = plist_.sublist("interfrost_dtheta_dpressure parameters");
  model_ = Teuchos::rcp(new InterfrostDthetaDpressureModel(sublist));
  InitializeFromPlist_();
}


// Copy constructor
InterfrostDthetaDpressureEvaluator::InterfrostDthetaDpressureEvaluator(const InterfrostDthetaDpressureEvaluator& other) :
    SecondaryVariableFieldEvaluator(other),
    nl_key_(other.nl_key_),
    sl_key_(other.sl_key_),
    phi_key_(other.phi_key_),    
    model_(other.model_) {}


// Virtual copy constructor
Teuchos::RCP<FieldEvaluator>
InterfrostDthetaDpressureEvaluator::Clone() const
{
  return Teuchos::rcp(new InterfrostDthetaDpressureEvaluator(*this));
}


// Initialize by setting up dependencies
void
InterfrostDthetaDpressureEvaluator::InitializeFromPlist_()
{
  // Set up my dependencies
  // - defaults to prefixed via domain
  Key domain_name = Keys::getDomainPrefix(my_key_);

  // - pull Keys from plist
  // dependency: molar_density_liquid
  nl_key_ = plist_.get<std::string>("molar density liquid key",
          domain_name+"molar_density_liquid");
  dependencies_.insert(nl_key_);

  // dependency: saturation_liquid
  sl_key_ = plist_.get<std::string>("saturation liquid key",
          domain_name+"saturation_liquid");
  dependencies_.insert(sl_key_);

  // dependency: porosity
  phi_key_ = plist_.get<std::string>("porosity key",
          domain_name+"porosity");
  dependencies_.insert(phi_key_);
}


void
InterfrostDthetaDpressureEvaluator::EvaluateField_(const Teuchos::Ptr<State>& S,
        const Teuchos::Ptr<CompositeVector>& result)
{
Teuchos::RCP<const CompositeVector> nl = S->GetFieldData(nl_key_);
Teuchos::RCP<const CompositeVector> sl = S->GetFieldData(sl_key_);
Teuchos::RCP<const CompositeVector> phi = S->GetFieldData(phi_key_);

  for (CompositeVector::name_iterator comp=result->begin();
       comp!=result->end(); ++comp) {
    const Epetra_MultiVector& nl_v = *nl->ViewComponent(*comp, false);
    const Epetra_MultiVector& sl_v = *sl->ViewComponent(*comp, false);
    const Epetra_MultiVector& phi_v = *phi->ViewComponent(*comp, false);
    Epetra_MultiVector& result_v = *result->ViewComponent(*comp,false);

    int ncomp = result->size(*comp, false);
    for (int i=0; i!=ncomp; ++i) {
      result_v[0][i] = model_->DThetaDpCoef(nl_v[0][i], sl_v[0][i], phi_v[0][i]);
    }
  }
}


void
InterfrostDthetaDpressureEvaluator::EvaluateFieldPartialDerivative_(const Teuchos::Ptr<State>& S,
        Key wrt_key, const Teuchos::Ptr<CompositeVector>& result)
{
Teuchos::RCP<const CompositeVector> nl = S->GetFieldData(nl_key_);
Teuchos::RCP<const CompositeVector> sl = S->GetFieldData(sl_key_);
Teuchos::RCP<const CompositeVector> phi = S->GetFieldData(phi_key_);

  if (wrt_key == nl_key_) {
    for (CompositeVector::name_iterator comp=result->begin();
         comp!=result->end(); ++comp) {
      const Epetra_MultiVector& nl_v = *nl->ViewComponent(*comp, false);
      const Epetra_MultiVector& sl_v = *sl->ViewComponent(*comp, false);
      const Epetra_MultiVector& phi_v = *phi->ViewComponent(*comp, false);
      Epetra_MultiVector& result_v = *result->ViewComponent(*comp,false);

      int ncomp = result->size(*comp, false);
      for (int i=0; i!=ncomp; ++i) {
        result_v[0][i] = model_->DDThetaDpCoefDMolarDensityLiquid(nl_v[0][i], sl_v[0][i], phi_v[0][i]);
      }
    }

  } else if (wrt_key == sl_key_) {
    for (CompositeVector::name_iterator comp=result->begin();
         comp!=result->end(); ++comp) {
      const Epetra_MultiVector& nl_v = *nl->ViewComponent(*comp, false);
      const Epetra_MultiVector& sl_v = *sl->ViewComponent(*comp, false);
      const Epetra_MultiVector& phi_v = *phi->ViewComponent(*comp, false);
      Epetra_MultiVector& result_v = *result->ViewComponent(*comp,false);

      int ncomp = result->size(*comp, false);
      for (int i=0; i!=ncomp; ++i) {
        result_v[0][i] = model_->DDThetaDpCoefDSaturationLiquid(nl_v[0][i], sl_v[0][i], phi_v[0][i]);
      }
    }

  } else if (wrt_key == phi_key_) {
    for (CompositeVector::name_iterator comp=result->begin();
         comp!=result->end(); ++comp) {
      const Epetra_MultiVector& nl_v = *nl->ViewComponent(*comp, false);
      const Epetra_MultiVector& sl_v = *sl->ViewComponent(*comp, false);
      const Epetra_MultiVector& phi_v = *phi->ViewComponent(*comp, false);
      Epetra_MultiVector& result_v = *result->ViewComponent(*comp,false);

      int ncomp = result->size(*comp, false);
      for (int i=0; i!=ncomp; ++i) {
        result_v[0][i] = model_->DDThetaDpCoefDPorosity(nl_v[0][i], sl_v[0][i], phi_v[0][i]);
      }
    }

  } else {
    AMANZI_ASSERT(0);
  }
}


} //namespace
} //namespace
} //namespace
